package net.jaekl.squelch;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Properties;

import net.jaekl.squelch.util.FileUtil;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class Args {
	private static final String DB_URL = "db.url";
	private static final String DB_PASSWORD = "db.password";
	private static final String DB_USER = "db.user";
	private static final String DEFAULT_CONFIG_FILENAME = "config.properties";
	
	private String m_className;	// name of the main class that launches this program
	private boolean m_help;
	private String m_url;
	private String m_pass;
	private String m_user;
	
	public Args(String className) {
		m_className = className;
		m_help = false;
		m_url = "";
		m_pass = "";
		m_user = "";
	}
	
	public boolean isHelp() { return m_help; }
	public String getUrl() { return m_url; }
	public String getPass() { return m_pass; }
	public String getUser() { return m_user; }
	
	public Options createOptions() 
	{
		Options opt = new Options();

		opt.addOption("c",  "config",      true,  "Config file");
		opt.addOption("h",  "help",        false, "Show usage information");
		opt.addOption("r",  "url",         true,  "DB JDBC URL");
		opt.addOption("p",  "password",    true,  "DB password");
		opt.addOption("u",  "user",        true,  "DB username");
		
		return opt;
	}

	public boolean parseArgs(PrintWriter pw, String[] args) throws IOException 
	{
		Options opt = createOptions();
		try {
			CommandLine line = new DefaultParser().parse(opt, args);
	
			String configFilename = (line.hasOption("c")) ? (line.getOptionValue("c")) : (DEFAULT_CONFIG_FILENAME);
			File configFile = FileUtil.getInst().newFile(configFilename);
			if (configFile.canRead()) {
				readConfigFile(configFile);
			}
			
			m_help   = line.hasOption("h");

			if (line.hasOption("p")) {
				m_pass = line.getOptionValue("p");
			}
			if (line.hasOption("r")) {
				m_url = line.getOptionValue("r");
			}
			if (line.hasOption("u")) {
				m_user = line.getOptionValue("u");
			}
		}
		catch (ParseException exc) {
			usage(pw, opt);
			return false;
		}
		
		if (m_help) {
			usage(pw, opt);
		}
		
		return true;
	}
	
	void readConfigFile(File configFile) throws IOException
	{
		Properties props = new Properties();		
		try (InputStream is = FileUtil.getInst().openStream(configFile)) {
			props.load(is);
		}
		
		m_url = props.getProperty(DB_URL, "");
		m_pass = props.getProperty(DB_PASSWORD, "");
		m_user = props.getProperty(DB_USER, "");
	}
	
	// Note that this leverages commons-cli's HelpFormatter to 
	// generate the usage message.  It will always be in English.
	// If we want to localize that, we'd need to recode this,
	// and also translate the parameter descriptions in 
	// createOptions().
	void usage(PrintWriter pw, Options opt) 
	{
		HelpFormatter help = new HelpFormatter();
		help.printHelp(pw, 80, m_className, "", opt, 0, 0, "", true);
	}
}
