package net.jaekl.squelch;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Locale;

import net.jaekl.squelch.stmt.Select;
import net.jaekl.squelch.stmt.Stmt;
import net.jaekl.squelch.util.ConsoleInput;
import net.jaekl.squelch.util.ConsoleInputImpl;
import net.jaekl.squelch.util.ConsoleUtil;

public class Squelch {
	static final String PROMPT = "> ";
	private static final Stmt[] READ_ONLY_STATEMENTS = {
		new Select()
	};
	
	private Stmt[] m_statements;

	public Squelch() {
		m_statements = READ_ONLY_STATEMENTS;
	}
	
	public void doMain(String[] args)
	{
		ConsoleInputImpl ci;
		try (PrintWriter pw = new PrintWriter(System.out))
		{
			ci = ConsoleUtil.getInst().getInput();
			pumpLines(pw, ci);			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		new Squelch().doMain(args);
	}
	
	void pumpLines(PrintWriter pw, ConsoleInput ci) throws IOException {
		String line = null;
		while (true) {
			boolean processed = false;
			line = ci.readLine(PROMPT);
			
			for (Stmt statement : m_statements) {
				if (statement.handles(line)) {
					//statement.exec(conn, pw, line);
					processed = true;
					break;
				}
			}
			
			if ((!processed)) {
				if (isQuit(line)) {
					break;
				}
				// Unrecognized command
				pw.println("??? \"" + line + "\"");
			}
		}
	}
	
	boolean isQuit(String line) {
		if ((null == line)) {
			return true;
		}
		String trimmed = line.trim();
		if (trimmed.endsWith(";")) {
			trimmed = trimmed.substring(0, trimmed.length() - 1).trim();
		}
		String upperCased = trimmed.toUpperCase(Locale.CANADA);
		
		if (  "EXIT".equals(upperCased)
		   || "QUIT".equals(upperCased)
		   || "\\q".equals(trimmed) ) 
		{
			return true;
		}
		return false;
	}
}
