package net.jaekl.squelch;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.nio.charset.StandardCharsets;

import net.jaekl.squelch.util.ConsoleInputMock;

import org.junit.Test;

public class SquelchTest {

	@Test
	public void test_isQuit() {
		Squelch squelch = new Squelch();
		
		final String[] AFFIRMATIVE = {
				"QUIT", " QUIT", " QUIT ", "\tQUIT\r\n", "\\q", "  \\q", "QUIT;", "Quit", "qUiT", " qUIt ", "quit ;",
				"exit", "ExIT", "EXIT", "  EXIT ; ", null
		};
		final String[] NEGATIVE = {
				"Fred", "SELECT * FROM FOO", "describe tablename", "",
				"CREATE TABLE Fred ( foo INTEGER );"
		};
		
		for (String s : AFFIRMATIVE) {
			assertTrue("isQuit(\"" + s + "\") should be true.", squelch.isQuit(s));
		}
		for (String s : NEGATIVE) {
			assertFalse("isQuit(\"" + s + "\") should be false.", squelch.isQuit(s));
		}
	}
	
	@Test
	public void test_pumpLines() throws IOException {
		Squelch squelch = new Squelch();
		String consoleOutput;
		
		String[] noLines = {};
		consoleOutput = runPump(squelch, noLines);
		assertEquals("", consoleOutput);
		
		String[] quitOnly = {"QUIT"};
		consoleOutput = runPump(squelch, quitOnly);
		assertEquals(Squelch.PROMPT + "QUIT\n", consoleOutput);
		
		String[] unrecognizedThenQuit = {"This is not a valid command;", "\\q"};
		consoleOutput = runPump(squelch, unrecognizedThenQuit);
		assertEquals(Squelch.PROMPT + "This is not a valid command;\n"
				     + "??? \"This is not a valid command;\"\n"
				     + Squelch.PROMPT + "\\q\n",
				     consoleOutput);
	}
	
	private String runPump(Squelch squelch, String[] lines) throws IOException
	{
		try (
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				PrintWriter pw = new PrintWriter(new OutputStreamWriter(baos, StandardCharsets.UTF_8))
			)
		{
			ConsoleInputMock cim = new ConsoleInputMock(pw, lines);
			squelch.pumpLines(pw, cim);
			pw.close();
			baos.close();
			return baos.toString();
		}		
	}
}
